/*
*  Name:         uesqlc.cc
*  Author:       Rafael Jesus Alcantara Perez
*  Summary:      UESQLC main
*  Date:         $Date: 2000/02/16 13:34:07 $
*  Revision:     $Revision: 1.10 $
*  Description:  Compiler for UESQL language.
*                UESQL is a subset of:
*
*                ISO/IEC 9075:1992, "Information Technology -- Database Languages -- SQL"
*                ANSI X3.135-1992,  "Database Language SQL"
*
*  Copyright (C) 1996-1999  Rafael Jesus Alcantara Perez <rafa@dedalo-ing.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <clocale>
#include "../../config.h"
#include "application.hh"


//
//  E X P O R T E D   V A R I A B L E S
//

///  Compiler application.
TApplication   tCompilerApp (PACKAGE, VERSION);


///  Main.
int main (int argc, const char* argv[])
{

  int   iExitCode;
  
  setlocale (LC_ALL, "");
  try
  {
    tCompilerApp.initialize (argc, argv);
    iExitCode = tCompilerApp.start();
  }
  catch (const mpcl::TExceptionCP_NotParameterAble& rktSOURCE_EXCEPTION)
  {
    //
    //  Exception:  Not parameter-able option.
    //
    iExitCode = 4;
    cerr << rktSOURCE_EXCEPTION.generalDescription();
    if ( !rktSOURCE_EXCEPTION.specificDescription().empty() )
    {
      cerr << " (--" << rktSOURCE_EXCEPTION.specificDescription() << ")";
    }
    cerr << endl;
  }
  catch (const mpcl::TBaseException& rktSOURCE_EXCEPTION)
  {
    iExitCode = 3;
    cerr << rktSOURCE_EXCEPTION;
  }
  catch (const std::exception& rktSOURCE_EXCEPTION)
  {
    iExitCode = 2;
    cerr << rktSOURCE_EXCEPTION.what() << endl;
  }
  catch (...)
  {
    iExitCode = 1;
    cerr << mpcl::TBaseException ("unhandled exception", NULL, __FILE__, __LINE__);
  }
  return iExitCode;
  
}  // main()
